//****************************************************************************
//                                        Denise Sciamarella & Gabriel Mindlin
//              Unveiling the topological structure of chaotic flows from data 
//                                        Physical Review E, 64, 036209, 2001.
//
//                                                     Rouen, October 31, 2019
//****************************************************************************
# include <cstdlib>
# include <iostream>
# include <fstream>
# include <iomanip>
# include <cmath>
# include <ctime>

using namespace std;

  const double a = 7.0;
  const double b = 1.45;
  const double c = 1.0;
  const double d = 0.5;
  const double R = 6.0;
  const double alpha = 0.3;
  const double beta = 7.0;
  const double epsilon1 = 0.165;
  const double epsilon2 = 0.01;
  const double epsilon3 = 2.0;
  const double mu = 1.543;
  const double dt = 0.001;                                         // Time step

  const int m = 4;                                    // state space dimension 
// **************** Denise 2D system *****************************************
#define fx(x,y,z,w) -(z+2.0)*d*(x - (a + epsilon3*(2 + w)))+(2.0-z)*(alpha*(x-2.0) - beta*y - alpha*(x-2.0)*((x-2.0)*(x-2.0)+y*y)/(R*R))
#define fy(x,y,z,w) -(z+2.0)*(y-b) + (2.0-z)*(beta*(x-2) +alpha*y - alpha*y*(((x-2.0)*(x-2.0)+y*y)/(R*R)))
#define fz(x,y,z,w) (4.0-z*z)*(z+2.0-mu*(x+2.0))/epsilon1 - c*z
#define fw(x,y,z,w) (4.0-z*z)*(z+2.0-mu*(x+2.0))/epsilon2 - c*z

void Initial(double *v);                                 // Initial conditions
void Runge(double *v);                             // Fourth-order Runge-Kutta

int main ( )
{
  int j;
  const int Ntrans = 1000000;
  const int Npt = Ntrans + 100000;                    // number of data points
  double *v; 

  v = new double[m];
//****************************************************************************
  ofstream data_12 ("denise4D.dat");

//Initial(v);
  v[0] = 0.2;
  v[1] = 0.2;
  v[2] = 0.2;
  v[3] = 0.2;
  for (j = 0; j < Npt; j++)
  {
    Runge(v);
    if (j > Ntrans && j % 4 == 0)
    {
       data_12 << v[0] << " " << v[1] << " " << v[2] << " " << v[3] << "\n";
/*     data_12 << fx(v[0],v[1],v[2],v[3]) << " " 
       << fy(v[0],v[1],v[2],v[3]) << " " 
       << fz(v[0],v[1],v[2],v[3]) << " " 
       << fw(v[0],v[1],v[2],v[3]) << "\n"; */
    }
  }
  data_12.close ( );
  system("xmgrace denise4D.dat -param denise4D.par"); 
  return 0;
}
// ***************************************************************************
   void Initial(double *v)                               // Initial conditions
// ***************************************************************************
   {
     int j;
     {
       for (j = 0; j < m; j++) 
       { 
         v[j]=1.0*drand48();
       }
     }
   }
// ***************************************************************************
   void Runge(double *v)                           // Fourth-order Runge-Kutta
// ***************************************************************************
   {
      int j;
      double phi[m], savev[m], FF[m];

      FF[0] = fx(v[0],v[1],v[2],v[3]);
      FF[1] = fy(v[0],v[1],v[2],v[3]);
      FF[2] = fz(v[0],v[1],v[2],v[3]);
      FF[3] = fw(v[0],v[1],v[2],v[3]);
      for (j=0; j <m; j++)
      {
        savev[j]=v[j];
        phi[j]=FF[j];
        v[j]=savev[j]+0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2],v[3]);
      FF[1] = fy(v[0],v[1],v[2],v[3]);
      FF[2] = fz(v[0],v[1],v[2],v[3]);
      FF[3] = fw(v[0],v[1],v[2],v[3]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j]=savev[j] + 0.5*dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2],v[3]);
      FF[1] = fy(v[0],v[1],v[2],v[3]);
      FF[2] = fz(v[0],v[1],v[2],v[3]);
      FF[3] = fw(v[0],v[1],v[2],v[3]);
      for (j=0; j <m; j++)
      {
        phi[j] = phi[j] + 2.0*FF[j];
        v[j] = savev[j] + dt*FF[j];
      }
      FF[0] = fx(v[0],v[1],v[2],v[3]);
      FF[1] = fy(v[0],v[1],v[2],v[3]);
      FF[2] = fz(v[0],v[1],v[2],v[3]);
      FF[3] = fw(v[0],v[1],v[2],v[3]);
      for (j=0; j <m; j++)
      {
        v[j] = savev[j] + (phi[j] + FF[j])*dt/6.0;
      }
   }
// ***************************************************************************
